@@ -200,7 +200,7 @@ superio_exit(void)
 #define W83627THF_REG_PWM2		0x03	/* 697HF and 637HF too */
 #define W83627THF_REG_PWM3		0x11	/* 637HF too */
 
-#define W83627THF_REG_VRM_OVT_CFG 	0x18	/* 637HF too, unused yet */
+#define W83627THF_REG_VRM_OVT_CFG 	0x18	/* 637HF too */
 
 static const u8 regpwm_627hf[] = { W83627HF_REG_PWM1, W83627HF_REG_PWM2 };
 static const u8 regpwm[] = { W83627THF_REG_PWM1, W83627THF_REG_PWM2,
@@ -225,6 +225,10 @@ static const u8 BIT_SCFG2[] = { 0x10, 0x20, 0x40 };
 #define IN_TO_REG(val)  (SENSORS_LIMIT((((val) * 10 + 8)/16),0,255))
 #define IN_FROM_REG(val) (((val) * 16 + 5) / 10)
 
+#define IN_TO_REG_VRM9(val) \
+	(SENSORS_LIMIT((((val) * 1000 - 70000 + 244) / 488), 0, 255))
+#define IN_FROM_REG_VRM9(reg)	(((reg) * 488 + 70000 + 500) / 1000)
+
 static inline u8 FAN_TO_REG(long rpm, int div)
 {
 	if (rpm == 0)
@@ -313,6 +317,7 @@ struct w83627hf_data {
 				   Default = 3435. 
 				   Other Betas unimplemented */
 	u8 vrm;
+	u8 vrm_ovt;		/* Register value, 627thf & 637hf only */
 };
 
 
@@ -848,25 +853,10 @@ static int w83627hf_write_value(struct i2c_client *client, u16 reg, u16 value)
 static void w83627hf_init_client(struct i2c_client *client)
 {
 	struct w83627hf_data *data = client->data;
-	int vid = 0, i;
+	int i;
 	int type = data->type;
 	u8 tmp;
 
-	if(init) {
-		/* save this register */
-		i = w83627hf_read_value(client, W83781D_REG_BEEP_CONFIG);
-		/* Reset all except Watchdog values and last conversion values
-		   This sets fan-divs to 2, among others */
-		w83627hf_write_value(client, W83781D_REG_CONFIG, 0x80);
-		/* Restore the register and disable power-on abnormal beep.
-		   This saves FAN 1/2/3 input/output values set by BIOS. */
-		w83627hf_write_value(client, W83781D_REG_BEEP_CONFIG, i | 0x80);
-		/* Disable master beep-enable (reset turns it on).
-		   Individual beeps should be reset to off but for some reason
-		   disabling this bit helps some people not get beeped */
-		w83627hf_write_value(client, W83781D_REG_BEEP_INTS2, 0);
-	}
-
 	/* Minimize conflicts with other winbond i2c-only clients...  */
 	/* disable i2c subclients... how to disable main i2c client?? */
 	/* force i2c address to relatively uncommon address */
@@ -882,10 +872,15 @@ static void w83627hf_init_client(struct i2c_client *client)
 		data->vid = w83627thf_read_gpio5(client) & 0x1f;
 	}
 
-	/* Convert VID to voltage based on default VRM */
-	data->vrm = DEFAULT_VRM;
-	if (type != w83697hf)
-		vid = vid_from_reg(vid, data->vrm);
+	/* Read VRM & OVT Config only once */
+	if (w83627thf == data->type || w83637hf == data->type)
+		data->vrm_ovt =
+			w83627hf_read_value(client, W83627THF_REG_VRM_OVT_CFG);
+	else
+		data->vrm_ovt = 0;
+
+	/* Choose VRM based on "VRM & OVT" register */
+	data->vrm = (data->vrm_ovt & 0x01) ? 90 : 82;
 
 	tmp = w83627hf_read_value(client, W83781D_REG_SCFG1);
 	for (i = 1; i <= 3; i++) {
@@ -1027,18 +1022,42 @@ void w83627hf_in(struct i2c_client *client, int operation, int ctl_name,
 		*nrels_mag = 2;
 	else if (operation == SENSORS_PROC_REAL_READ) {
 		w83627hf_update_client(client);
-		results[0] = IN_FROM_REG(data->in_min[nr]);
-		results[1] = IN_FROM_REG(data->in_max[nr]);
-		results[2] = IN_FROM_REG(data->in[nr]);
+
+		if (nr == 0 && (data->vrm_ovt & 0x01)) {
+			/* use VRM9 calculation */
+			results[0] = IN_FROM_REG_VRM9(data->in_min[0]);
+			results[1] = IN_FROM_REG_VRM9(data->in_max[0]);
+			results[2] = IN_FROM_REG_VRM9(data->in[0]);
+
+		} else {
+			/* use VRM8 (standard) calculation */
+			results[0] = IN_FROM_REG(data->in_min[nr]);
+			results[1] = IN_FROM_REG(data->in_max[nr]);
+			results[2] = IN_FROM_REG(data->in[nr]);
+		}
+
 		*nrels_mag = 3;
+
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
 		if (*nrels_mag >= 1) {
-			data->in_min[nr] = IN_TO_REG(results[0]);
+			if (nr == 0 && (data->vrm_ovt & 0x01))
+				/* use VRM9 calculation */
+				data->in_min[0] = IN_TO_REG_VRM9(results[0]);
+			else
+				/* use VRM8 (standard) calculation */
+				data->in_min[nr] = IN_TO_REG(results[0]);
+
 			w83627hf_write_value(client, W83781D_REG_IN_MIN(nr),
 					    data->in_min[nr]);
 		}
 		if (*nrels_mag >= 2) {
-			data->in_max[nr] = IN_TO_REG(results[1]);
+			if (nr == 0 && (data->vrm_ovt & 0x01))
+				/* use VRM9 calculation */
+				data->in_min[0] = IN_TO_REG_VRM9(results[1]);
+			else
+				/* use VRM8 (standard) calculation */
+				data->in_max[nr] = IN_TO_REG(results[1]);
+
 			w83627hf_write_value(client, W83781D_REG_IN_MAX(nr),
 					    data->in_max[nr]);
 		}
