@@ -475,11 +475,13 @@
 #define SENSORS_LM85_ZONE3_RANGE           100  /* RW -- zone3_range  */
 #define SENSORS_LM85_ZONE3_SMOOTH          101  /* RW -- zone3_smooth  */
 
-/* LM89/LM90/LM99/ADM1032 chips */
+/* LM86/LM89/LM90/LM99/ADM1032/MAX6657 chips */
 
 #define SENSORS_LM90_PREFIX "lm90"
 #define SENSORS_ADM1032_PREFIX "adm1032"
 #define SENSORS_LM99_PREFIX "lm99"
+#define SENSORS_LM86_PREFIX "lm86"
+#define SENSORS_MAX6657_PREFIX "max6657"
 
 #define SENSORS_LM90_LOCAL_TEMP 51 /* R */
 #define SENSORS_LM90_LOCAL_HIGH 52 /* RW */
@@ -1397,6 +1399,7 @@
 /* IT87xx chips */
 
 #define SENSORS_IT87_PREFIX "it87"
+#define SENSORS_IT8712_PREFIX "it8712"
 
 #define SENSORS_IT87_IN0 1 /* R */
 #define SENSORS_IT87_IN1 2 /* R */
@@ -1749,6 +1752,9 @@
 #define SENSORS_PC87360_TEMP5_STATUS	215	/* R */
 #define SENSORS_PC87360_TEMP6_STATUS	216	/* R */
 
+#define SENSORS_PC87360_VID		240	/* R */
+#define SENSORS_PC87360_VRM		241	/* RW */
+
 #define SENSORS_LM92_PREFIX "lm92"
 
 #define SENSORS_LM92_TEMP_HIGH		1	/* RW */
@@ -1830,6 +1836,77 @@
 #define SENSORS_BMC_CURR1_MIN 3101 /* RW */
 #define SENSORS_BMC_CURR1_MAX 3201 /* RW */
 
+#define SENSORS_LM93_PREFIX "lm93"
+
+#define SENSORS_LM93_IN1		1011
+#define SENSORS_LM93_IN1_MIN		1012
+#define SENSORS_LM93_IN1_MAX		1013
+#define SENSORS_LM93_IN2		1021
+#define SENSORS_LM93_IN2_MIN		1022
+#define SENSORS_LM93_IN2_MAX		1023
+#define SENSORS_LM93_IN3		1031
+#define SENSORS_LM93_IN3_MIN		1032
+#define SENSORS_LM93_IN3_MAX		1033
+#define SENSORS_LM93_IN4		1041
+#define SENSORS_LM93_IN4_MIN		1042
+#define SENSORS_LM93_IN4_MAX		1043
+#define SENSORS_LM93_IN5		1051
+#define SENSORS_LM93_IN5_MIN		1052
+#define SENSORS_LM93_IN5_MAX		1053
+#define SENSORS_LM93_IN6		1061
+#define SENSORS_LM93_IN6_MIN		1062
+#define SENSORS_LM93_IN6_MAX		1063
+#define SENSORS_LM93_IN7		1071
+#define SENSORS_LM93_IN7_MIN		1072
+#define SENSORS_LM93_IN7_MAX		1073
+#define SENSORS_LM93_IN8		1081
+#define SENSORS_LM93_IN8_MIN		1082
+#define SENSORS_LM93_IN8_MAX		1083
+#define SENSORS_LM93_IN9		1091
+#define SENSORS_LM93_IN9_MIN		1092
+#define SENSORS_LM93_IN9_MAX		1093
+#define SENSORS_LM93_IN10		1101
+#define SENSORS_LM93_IN10_MIN		1102
+#define SENSORS_LM93_IN10_MAX		1103
+#define SENSORS_LM93_IN11		1111
+#define SENSORS_LM93_IN11_MIN		1112
+#define SENSORS_LM93_IN11_MAX		1113
+#define SENSORS_LM93_IN12		1121
+#define SENSORS_LM93_IN12_MIN		1122
+#define SENSORS_LM93_IN12_MAX		1123
+#define SENSORS_LM93_IN13		1131
+#define SENSORS_LM93_IN13_MIN		1132
+#define SENSORS_LM93_IN13_MAX		1133
+#define SENSORS_LM93_IN14		1141
+#define SENSORS_LM93_IN14_MIN		1142
+#define SENSORS_LM93_IN14_MAX		1143
+#define SENSORS_LM93_IN15		1151
+#define SENSORS_LM93_IN15_MIN		1152
+#define SENSORS_LM93_IN15_MAX		1153
+#define SENSORS_LM93_IN16		1161
+#define SENSORS_LM93_IN16_MIN		1162
+#define SENSORS_LM93_IN16_MAX		1163
+#define SENSORS_LM93_TEMP1		2011
+#define SENSORS_LM93_TEMP1_MIN		2012
+#define SENSORS_LM93_TEMP1_MAX		2013
+#define SENSORS_LM93_TEMP2		2021
+#define SENSORS_LM93_TEMP2_MIN		2022
+#define SENSORS_LM93_TEMP2_MAX		2023
+#define SENSORS_LM93_TEMP3		2031
+#define SENSORS_LM93_TEMP3_MIN		2032
+#define SENSORS_LM93_TEMP3_MAX		2033
+#define SENSORS_LM93_FAN1		3011
+#define SENSORS_LM93_FAN1_MIN		3012
+#define SENSORS_LM93_FAN2		3021
+#define SENSORS_LM93_FAN2_MIN		3022
+#define SENSORS_LM93_FAN3		3031
+#define SENSORS_LM93_FAN3_MIN		3032
+#define SENSORS_LM93_FAN4		3041
+#define SENSORS_LM93_FAN4_MIN		3042
+#define SENSORS_LM93_VID1		4001
+#define SENSORS_LM93_VID2		4002
+#define SENSORS_LM93_ALARMS		5001
+
 #define SENSORS_ASB100_PREFIX "asb100"
 
 #define SENSORS_ASB100_IN0		0x01 /* R */
